@@ -1,6 +1,5 @@
 import json
 import os
-from pprint import pprint
 
 import click
 from pygments import highlight, lexers, formatters
@@ -8,25 +7,32 @@ from pygments import highlight, lexers, formatters
 from pymobiledevice3.lockdown import LockdownClient
 
 
-def print_object(buf, colored=True, default=None):
+def default_json_encoder(obj):
+    if isinstance(obj, bytes):
+        return obj.hex()
+    raise TypeError()
+
+
+def print_json(buf, colored=True, default=default_json_encoder):
+    formatted_json = json.dumps(buf, sort_keys=True, indent=4, default=default)
     if colored:
-        formatted_json = json.dumps(buf, sort_keys=True, indent=4, default=default)
-        colorful_json = highlight(formatted_json, lexers.JsonLexer(), formatters.TerminalFormatter())
+        colorful_json = highlight(formatted_json, lexers.JsonLexer(),
+                                  formatters.TerminalTrueColorFormatter(style='stata-dark'))
         print(colorful_json)
     else:
-        pprint(buf)
+        print(formatted_json)
 
 
 class Command(click.Command):
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.params[:0] = [
+            click.Option(('lockdown', '--udid'), callback=self.udid),
+        ]
+
     @staticmethod
     def udid(ctx, param, value):
         if '_PYMOBILEDEVICE3_COMPLETE' in os.environ:
             # prevent lockdown connection establishment when in autocomplete mode
             return
         return LockdownClient(udid=value)
-
-    def __init__(self, *args, **kwargs):
-        super(Command, self).__init__(*args, **kwargs)
-        self.params[:0] = [
-            click.Option(('lockdown', '--udid'), callback=self.udid),
-        ]
