@@ -3,19 +3,18 @@ import logging
 import os
 import platform
 import plistlib
-import re
 import sys
 import uuid
+from distutils.version import LooseVersion
 
 from pymobiledevice3 import usbmux
 from pymobiledevice3.ca import ca_do_everything
 from pymobiledevice3.exceptions import NoDeviceConnectedError, FatalPairingError, CannotStopSessionError, \
     NotTrustedError, \
     PairingError, NotPairedError, StartServiceError, DeviceNonConnectedError, PyMobileDevice3Exception, \
-    PasswordRequiredError
+    PasswordRequiredError, ConnectionFailedError
 from pymobiledevice3.service_connection import ServiceConnection
 
-
 # we store pairing records and ssl keys in ~/.pymobiledevice3
 HOMEFOLDER = '.pymobiledevice3'
 MAXTRIES = 20
@@ -107,32 +106,6 @@ class LockdownClient(object):
         self.paired = True
         return
 
-    def compare_ios_version(self, ios_version):
-        """
-        currrent_version > ios_version return 1
-        currrent_version = ios_version return 0
-        currrent_version < ios_version return -1
-        :param ios_version:
-        :return: int
-        """
-        version_reg = r'^\d*\.\d*\.?\d*$'
-        if not re.match(version_reg, ios_version):
-            raise PyMobileDevice3Exception('ios_version invalid:%s' % ios_version)
-        a = self.ios_version.split('.')
-        b = ios_version.split('.')
-        length = min(len(a), len(b))
-        for i in range(length):
-            if int(a[i]) < int(b[i]):
-                return -1
-            if int(a[i]) > int(b[i]):
-                return 1
-        if len(a) > len(b):
-            return 1
-        elif len(a) < len(b):
-            return -1
-        else:
-            return 0
-
     def query_type(self):
         self.service.send_plist({'Request': 'QueryType'})
         res = self.service.recv_plist()
@@ -168,13 +141,14 @@ class LockdownClient(object):
 
         filename = os.path.join(folder, f'{self.identifier}.plist')
 
-        if os.path.exists(filename):
+        try:
             with open(filename, 'rb') as f:
                 pair_record = plistlib.load(f)
             logging.warning(f'Using iTunes pair record: {self.identifier}.plist')
-        else:
+
+        except (PermissionError, FileNotFoundError):
             logging.warning(f'No iTunes pairing record found for device {self.identifier}')
-            if self.compare_ios_version('13.0') >= 0:
+            if LooseVersion(self.ios_version) >= LooseVersion('13.0'):
                 self.logger.warning('Getting pair record from usbmuxd')
                 client = usbmux.UsbmuxdClient()
                 pair_record = client.get_pair_record(self.udid)
@@ -193,7 +167,7 @@ class LockdownClient(object):
         cert_pem = pair_record['HostCertificate']
         private_key_pem = pair_record['HostPrivateKey']
 
-        if self.compare_ios_version('11.0') < 0:
+        if LooseVersion(self.ios_version) < LooseVersion('11.0'):
             validate_pair = {'Label': self.label, 'Request': 'ValidatePair', 'PairRecord': pair_record}
             self.service.send_plist(validate_pair)
             r = self.service.recv_plist()
@@ -284,7 +258,7 @@ class LockdownClient(object):
         self.logger.debug(response)
         return response
 
-    def start_service(self, name, ssl=False, escrow_bag=None) -> ServiceConnection:
+    def start_service(self, name, escrow_bag=None) -> ServiceConnection:
         if not self.paired:
             self.logger.info('NotPaired')
             raise NotPairedError()
@@ -295,7 +269,7 @@ class LockdownClient(object):
 
         self.service.send_plist(request)
         response = self.service.recv_plist()
-        ssl_enabled = response.get('EnableServiceSSL', ssl)
+        ssl_enabled = response.get('EnableServiceSSL', False)
         if not response or response.get('Error'):
             if response.get('Error', '') == 'PasswordProtected':
                 raise PasswordRequiredError(
@@ -305,3 +279,13 @@ class LockdownClient(object):
         if ssl_enabled:
             service_connection.ssl_start(self.ssl_file, self.ssl_file)
         return service_connection
+
+    def start_developer_service(self, name, escrow_bag=None) -> ServiceConnection:
+        try:
+            return self.start_service(name, escrow_bag)
+        except (StartServiceError, ConnectionFailedError):
+            self.logger.error(
+                'Failed to connect to required service. Make sure DeveloperDiskImage.dmg has been mounted. '
+                'You can do so using: pymobiledevice3 mounter mount'
+            )
+            raise
