@@ -1,15 +1,77 @@
 import typing
-from datetime import datetime
+import struct
+from datetime import datetime, timezone, timedelta
 import enum
 import uuid
 from collections import namedtuple
 
 from construct import Struct, Int32ul, Int64ul, FixedSized, GreedyRange, GreedyBytes, Enum, Switch, Padding, Padded, \
-    LazyBound, CString, Computed, Array, this, Byte, Int16ul, Pass, Const, Bytes, RawCopy
+    LazyBound, CString, Computed, Array, this, Byte, Int16ul, Pass, Const, Bytes, Adapter
 
-from pymobiledevice3.services.dvt.tap import Tap
+from pymobiledevice3.services.remote_server import Tap
+from pymobiledevice3.services.dvt.instruments.device_info import DeviceInfo
 from pymobiledevice3.services.dvt.dvt_secure_socket_proxy import DvtSecureSocketProxyService
 
+try:
+    from functools import cached_property
+except ImportError:
+
+    from threading import RLock
+
+    GenericAlias = type(list[int])
+
+    _NOT_FOUND = object()
+
+
+    class cached_property:
+        def __init__(self, func):
+            self.func = func
+            self.attrname = None
+            self.__doc__ = func.__doc__
+            self.lock = RLock()
+
+        def __set_name__(self, owner, name):
+            if self.attrname is None:
+                self.attrname = name
+            elif name != self.attrname:
+                raise TypeError(
+                    "Cannot assign the same cached_property to two different names "
+                    f"({self.attrname!r} and {name!r})."
+                )
+
+        def __get__(self, instance, owner=None):
+            if instance is None:
+                return self
+            if self.attrname is None:
+                raise TypeError(
+                    "Cannot use cached_property instance without calling __set_name__ on it.")
+            try:
+                cache = instance.__dict__
+            except AttributeError:  # not all objects have __dict__ (e.g. class defines slots)
+                msg = (
+                    f"No '__dict__' attribute on {type(instance).__name__!r} "
+                    f"instance to cache {self.attrname!r} property."
+                )
+                raise TypeError(msg) from None
+            val = cache.get(self.attrname, _NOT_FOUND)
+            if val is _NOT_FOUND:
+                with self.lock:
+                    # check if another thread filled cache while we awaited lock
+                    val = cache.get(self.attrname, _NOT_FOUND)
+                    if val is _NOT_FOUND:
+                        val = self.func(instance)
+                        try:
+                            cache[self.attrname] = val
+                        except TypeError:
+                            msg = (
+                                f"The '__dict__' attribute on {type(instance).__name__!r} instance "
+                                f"does not support item assignment for caching {self.attrname!r} property."
+                            )
+                            raise TypeError(msg) from None
+            return val
+
+        __class_getitem__ = classmethod(GenericAlias)
+
 # bsd/sys/kdebug.h
 RAW_VERSION2_BYTES = b'\x00\x02\xaa\x55'
 KDBG_CLASS_MASK = 0xff000000
@@ -188,6 +250,7 @@ predefined_names = {
     kcdata_types_enum.STACKSHOT_KCTYPE_THREAD_DISPATCH_QUEUE_LABEL: 'dispatch_queue_label',
     kcdata_types_enum.KCDATA_BUFFER_BEGIN_STACKSHOT: 'kcdata_stackshot',
     kcdata_types_enum.STACKSHOT_KCTYPE_STACKSHOT_FAULT_STATS: 'stackshot_fault_stats',
+    kcdata_types_enum.STACKSHOT_KCTYPE_STACKSHOT_DURATION: 'stackshot_duration',
 }
 
 predefined_name_substruct = 'name' / Computed(lambda ctx: predefined_names[ctx._.type])
@@ -407,6 +470,14 @@ stackshot_fault_stats = Struct(
     )
 )
 
+stackshot_duration = Struct(
+    predefined_name_substruct,
+    'obj' / Struct(
+        'stackshot_duration' / Int64ul,
+        'stackshot_duration_outer' / Int64ul,
+    )
+)
+
 kcdata_types_structures = {
     kcdata_types_enum.KCDATA_TYPE_UINT32_DESC: uint32_desc,
     kcdata_types_enum.KCDATA_TYPE_UINT64_DESC: uint64_desc,
@@ -444,6 +515,7 @@ kcdata_types_structures = {
     kcdata_types_enum.KCDATA_BUFFER_BEGIN_STACKSHOT: Struct(predefined_name_substruct),
     kcdata_types_enum.KCDATA_TYPE_CONTAINER_END: Pass,
     kcdata_types_enum.KCDATA_TYPE_BUFFER_END: Pass,
+    kcdata_types_enum.STACKSHOT_KCTYPE_STACKSHOT_DURATION: stackshot_duration,
 }
 
 kcdata_item = Struct(
@@ -463,13 +535,15 @@ kd_threadmap = Struct(
     'process' / FixedSized(20, CString('utf8')),
 )
 
+"""
+The constrcut version of kdebug event is:
 kd_buf = Struct(
     'timestamp' / Int64ul,
     'args' / RawCopy(Array(4, Int64ul)),
     'tid' / Int64ul,
     'debugid' / Int32ul,
     'eventid' / Computed(lambda ctx: ctx.debugid & KDBG_EVENTID_MASK),
-    'class' / Computed(lambda ctx: (ctx.debugid & KDBG_CLASS_MASK) >> KDBG_CLASS_OFFSET),
+    'class_' / Computed(lambda ctx: (ctx.debugid & KDBG_CLASS_MASK) >> KDBG_CLASS_OFFSET),
     'subclass' / Computed(lambda ctx: (ctx.debugid & KDBG_SUBCLASS_MASK) >> KDBG_SUBCLASS_OFFSET),
     'code' / Computed(lambda ctx: (ctx.debugid & KDBG_CODE_MASK) >> KDBG_CODE_OFFSET),
     'func_qualifier' / Computed(lambda ctx: ctx.debugid & KDBG_FUNC_MASK),
@@ -477,6 +551,19 @@ kd_buf = Struct(
     'unused' / Int64ul,
 )
 
+Unfortunately, it is too slow.
+"""
+
+kd_buf_format = '<Q32sQIIQ'
+kd_buf_size = struct.calcsize(kd_buf_format)
+KdBuf = namedtuple('KdBuf', ['timestamp', 'data', 'values', 'tid', 'debugid', 'eventid', 'func_qualifier'])
+
+
+class KdBufAdapter(Adapter):
+    def _decode(self, obj, context, path):
+        return make_event(obj)
+
+
 kperf_data = Struct(
     'magic' / Const(RAW_VERSION2_BYTES, Bytes(4)),
     'number_of_treads' / Int32ul,
@@ -487,7 +574,7 @@ kperf_data = Struct(
     Padding(0x100),
     'threadmap' / Array(lambda ctx: ctx.number_of_treads, kd_threadmap),
     '_pad' / GreedyRange(Const(0, Byte)),
-    'traces' / GreedyRange(kd_buf)
+    'traces' / GreedyRange(KdBufAdapter(Bytes(kd_buf_size)))
 )
 
 
@@ -527,6 +614,13 @@ def jsonify_parsed_stackshot(stackshot, root=None, index=0):
             root[item['data']['name']] = item['data']['obj']
 
 
+def make_event(buf):
+    timestamp, args_buf, tid, debugid, cpuid, unused = struct.unpack(kd_buf_format, buf)
+    eventid = debugid & KDBG_EVENTID_MASK
+    qual = debugid & KDBG_FUNC_MASK
+    return KdBuf(timestamp, args_buf, struct.unpack('<QQQQ', args_buf), tid, debugid, eventid, qual)
+
+
 ProcessData = namedtuple('namedtuple', ['pid', 'name'])
 
 
@@ -534,6 +628,7 @@ class DgbFuncQual(enum.Enum):
     DBG_FUNC_NONE = 0
     DBG_FUNC_START = 1
     DBG_FUNC_END = 2
+    DBG_FUNC_ALL = 3
 
 
 class CoreProfileSessionTap(Tap):
@@ -563,27 +658,25 @@ class CoreProfileSessionTap(Tap):
     IDENTIFIER = 'com.apple.instruments.server.services.coreprofilesessiontap'
     STACKSHOT_HEADER = Int32ul.build(int(kcdata_types_enum.KCDATA_BUFFER_BEGIN_STACKSHOT))
 
-    def __init__(self, dvt: DvtSecureSocketProxyService, class_filter: int = None, subclass_filter: int = None):
+    def __init__(self, dvt: DvtSecureSocketProxyService, time_config: typing.Mapping, filters: typing.Set = None):
         """
         :param dvt: Instruments service proxy.
-        :param class_filter: Event class to include.
-        :param subclass_filter: Event subclass to include.
+        :param time_config: Timing information - numer, denom, mach_absolute_time and matching usecs_since_epoch,
+        timezone.
+        :param filters: Event filters to include, Include all if empty.
         """
         self.dvt = dvt
         self.stack_shot = None
         self._thread_map = {}
         self.uuid = str(uuid.uuid4())
+        self.time_config = time_config
 
-        k_filter = 0xffffffff
-        if class_filter is not None:
-            k_filter = class_filter << KDBG_CLASS_OFFSET
-        if subclass_filter is not None:
-            k_filter |= subclass_filter << KDBG_SUBCLASS_OFFSET
-
+        if filters is None:
+            filters = {0xffffffff}
         config = {
             'tc': [{
                 'csd': 128,  # Callstack frame depth.
-                'kdf2': {k_filter},  # Kdebug filter, receive all classes.
+                'kdf2': filters,  # Kdebug filter, receive all classes.
                 'ta': [[3], [0], [2], [1, 1, 0]],  # Actions.
                 'tk': 3,  # Kind.
                 'uuid': self.uuid,
@@ -599,7 +692,7 @@ class CoreProfileSessionTap(Tap):
 
     @thread_map.setter
     def thread_map(self, parsed_threadmap):
-        self._thread_map = {}
+        self._thread_map.clear()
         for thread in parsed_threadmap:
             self._thread_map[thread.tid] = ProcessData(thread.pid, thread.process)
 
@@ -649,14 +742,18 @@ class CoreProfileSessionTap(Tap):
                 parsed = kperf_data.parse(data)
                 self.thread_map = parsed.threadmap
                 traces = parsed.traces
+                for event in traces:
+                    if events_index == events_count:
+                        break
+                    yield event
+                    events_index += 1
             else:
-                traces = Array(len(data) // kd_buf.sizeof(), kd_buf).parse(data)
-
-            for event in traces:
-                if events_index == events_count:
-                    break
-                yield event
-                events_index += 1
+                for i in range(int(len(data) / kd_buf_size)):
+                    buf = data[i * kd_buf_size: (i + 1) * kd_buf_size]
+                    if events_index == events_count:
+                        break
+                    yield make_event(buf)
+                    events_index += 1
 
     @staticmethod
     def parse_stackshot(data):
@@ -667,10 +764,37 @@ class CoreProfileSessionTap(Tap):
         jsonify_parsed_stackshot(stackshot, parsed_stack_shot)
         return parsed_stack_shot[predefined_names[kcdata_types_enum.KCDATA_BUFFER_BEGIN_STACKSHOT]]
 
-    def parse_event_time(self, timestamp):
-        time_info = self.stack_shot['mach_timebase_info']
-        offset_usec = (
-                ((timestamp - self.stack_shot['mach_absolute_time']) * time_info['numer']) /
-                (time_info['denom'] * 1000)
+    @staticmethod
+    def get_time_config(dvt):
+        mach_absolute_time, numer, denom, _ = DeviceInfo(dvt).mach_time_info()
+        usecs_since_epoch = dvt.lockdown.get_value(key='TimeIntervalSince1970') * 1000000
+        return dict(
+            numer=numer, denom=denom, mach_absolute_time=mach_absolute_time, usecs_since_epoch=usecs_since_epoch,
+            timezone=timezone(timedelta(seconds=dvt.lockdown.get_value(key='TimeZoneOffsetFromUTC')))
         )
-        return datetime.fromtimestamp((self.stack_shot['usecs_since_epoch'] + offset_usec) / 1000000)
+
+    def parse_event_time(self, timestamp) -> datetime:
+        offset_usec = (
+                ((timestamp - self.mach_absolute_time) * self.numer) / self.denom
+        )
+        return datetime.fromtimestamp((self.usecs_since_epoch + offset_usec) / 1000000, tz=self.timezone)
+
+    @cached_property
+    def numer(self):
+        return self.time_config['numer']
+
+    @cached_property
+    def denom(self):
+        return self.time_config['denom'] * 1000
+
+    @cached_property
+    def mach_absolute_time(self):
+        return self.time_config['mach_absolute_time']
+
+    @cached_property
+    def usecs_since_epoch(self):
+        return self.time_config['usecs_since_epoch']
+
+    @cached_property
+    def timezone(self):
+        return self.time_config['timezone']
